import Popover from '@vue-cdk/popover/src/popover';
import Vue from 'vue';

// @ts-check

/** @type {import('./types/options').GetComponentName} */
// By default we prefix all of our components with a 'C'. A prefix is not needed in 99% of all cases but there are some situations which require a prefix. One edge case is the 'Link' component. 'Link' is considered by some players in the Vue ecosystem to be an invalid component name. For example having the following component:

/*
<template>
  <CLink>hi</CLink>
</template>
*/
// Will make eslint complain ('The template root requires an element rather than texts' and 'Parsing error: x-invalid-end-tag'). Because of that we simply prefix all of our components with a 'C'.
const defaultComponentName = context => `C${context.name}`; // @ts-check

/** @returns {import('./types/options')._Options} */


var createDefaultOptions = () => ({
  onDidRegisterComponent: () => {},
  componentName: defaultComponentName
}); // @ts-check


var normalizedPluginOptions = (options = createDefaultOptions()) => {
  return { ...createDefaultOptions(),
    ...options
  };
}; // @ts-check


const getComponentName = component => {
  if (typeof component === 'function') {
    const {
      prototype
    } = component;
    return prototype.constructor.extendOptions.name;
  }

  return component.name;
}; // This function turns any constructor into a Vue plugin.
// If you have a components that depends on other components to be globally
// installed pass them as the seoncond parameter.

/** @param {Array<import('vue').Component>} dependencies */


var index = (...dependencies) => {
  const install = (vue, options) => {
    const _options = normalizedPluginOptions(options);

    const {
      onDidRegisterComponent
    } = _options;
    dependencies.forEach(component => {
      const rawComponentName = getComponentName(component);

      if (rawComponentName == null) {
        throw new Error(`
            Unable to determine component name. Component: ${component}. Did you forget to add a 'name' attribute?
                `);
      }

      const componentName = _options.componentName({
        name: rawComponentName,
        component: component
      });

      vue.component(componentName, component);
      onDidRegisterComponent({
        component,
        name: componentName
      });
    });
  };

  function createMixin(options) {
    const _options = normalizedPluginOptions(options);

    const componentsOption = {};
    dependencies.forEach(component => {
      const rawComponentName = getComponentName(component);

      if (rawComponentName == null) {
        throw new Error(`
            Unable to determine component name. Component: ${component}. Did you forget to add a 'name' attribute?
                `);
      }

      const componentName = _options.componentName({
        name: rawComponentName,
        component: component
      });

      componentsOption[componentName] = component;
    });
    return {
      components: componentsOption
    };
  }

  createMixin.install = install;
  return createMixin;
};

// @ts-check

/**
 * @template {keyof DocumentEventMap} K
 * @param {K} type
 * @param {HTMLElement | Document} subject
 * @param {(event: DocumentEventMap[K]) => void} handler
 */
const createEventListener = (type, subject, handler) => {
  let isOn = false;
  return {
    isOn: () => isOn,

    on() {
      if (isOn) {
        throw Error('cannot enable a listener that is already enabled');
      }

      isOn = true;
      subject.addEventListener(type, handler);
      return this;
    },

    off() {
      if (isOn === false) {
        throw Error('cannot disable a listener that is not enabled');
      }

      isOn = false;
      subject.removeEventListener(type, handler);
      return this;
    }

  };
};

// @ts-check
/**
 * @param {() => {}} callback
 * @param {{element?: HTMLElement }} options
 */

var createOnEscListener = ((callback, options = {}) => {
  const element = options.element || document;
  return createEventListener('keyup', element, event => {
    if (event.key === 'Escape') {
      callback();
    }
  });
});

//
//
//

var script = {
  name: 'PopoverArrow',
};

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier
/* server only */
, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
  if (typeof shadowMode !== 'boolean') {
    createInjectorSSR = createInjector;
    createInjector = shadowMode;
    shadowMode = false;
  } // Vue.extend constructor export interop.


  const options = typeof script === 'function' ? script.options : script; // render functions

  if (template && template.render) {
    options.render = template.render;
    options.staticRenderFns = template.staticRenderFns;
    options._compiled = true; // functional template

    if (isFunctionalTemplate) {
      options.functional = true;
    }
  } // scopedId


  if (scopeId) {
    options._scopeId = scopeId;
  }

  let hook;

  if (moduleIdentifier) {
    // server build
    hook = function (context) {
      // 2.3 injection
      context = context || // cached call
      this.$vnode && this.$vnode.ssrContext || // stateful
      this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext; // functional
      // 2.2 with runInNewContext: true

      if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
        context = __VUE_SSR_CONTEXT__;
      } // inject component styles


      if (style) {
        style.call(this, createInjectorSSR(context));
      } // register component module identifier for async chunk inference


      if (context && context._registeredComponents) {
        context._registeredComponents.add(moduleIdentifier);
      }
    }; // used by ssr in case component is cached and beforeCreate
    // never gets called


    options._ssrRegister = hook;
  } else if (style) {
    hook = shadowMode ? function (context) {
      style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
    } : function (context) {
      style.call(this, createInjector(context));
    };
  }

  if (hook) {
    if (options.functional) {
      // register for functional component in vue file
      const originalRender = options.render;

      options.render = function renderWithStyleInjection(h, context) {
        hook.call(context);
        return originalRender(h, context);
      };
    } else {
      // inject component registration as beforeCreate hook
      const existing = options.beforeCreate;
      options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
    }
  }

  return script;
}

/* script */
const __vue_script__ = script;

/* template */
var __vue_render__ = function() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c("div", { attrs: { "data-popper-arrow": "true" } })
};
var __vue_staticRenderFns__ = [];
__vue_render__._withStripped = true;

  /* style */
  const __vue_inject_styles__ = undefined;
  /* scoped */
  const __vue_scope_id__ = undefined;
  /* module identifier */
  const __vue_module_identifier__ = undefined;
  /* functional template */
  const __vue_is_functional_template__ = false;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__ = /*#__PURE__*/normalizeComponent(
    { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    false,
    undefined,
    undefined,
    undefined
  );

// @ts-check

/** @typedef {string | null | undefined} Value */

/**
 * @param {any} cls
 * @returns {boolean}
 */
const invalidClasses = cls => typeof cls === 'string' && cls.length > 0;
/**
 * @param {Array.<Value>} classes
 * @returns {Array.<string> | null}
 */


const _classes = classes => {
  // Filter out null, undefined and ""
  const onlyClasses = classes.filter(invalidClasses);

  if (onlyClasses.length === 0) {
    return null;
  } // @ts-ignore


  return onlyClasses;
};

//

var script$1 = {
  name: 'PopoverContent',
  components: { CPopoverArrow: __vue_component__ },
  inject: {
    $_popover: {
      default: {
        theme: null,
        withArrow: false,
        arrowClasses: [],
      },
    },
  },
  computed: {
    injectedArrowClasses() {
      return this.$_popover.arrowClasses
    },
    arrowClasses() {
      const { theme, injectedArrowClasses } = this;
      return _classes([injectedArrowClasses, theme ? 'vcdk-popover--arrow' : null])
    },
    theme() {
      return this.$_popover.theme
    },
    classes() {
      const { theme, withArrow } = this;
      return _classes([
        'vcdk-popover-body',
        theme ? `vcdk-popover-theme-${this.theme}` : null,
        !withArrow ? 'vcdk-popover--no-arrow' : null,
      ])
    },
    withArrow() {
      return this.$_popover.withArrow
    },
  },
};

/* script */
const __vue_script__$1 = script$1;

/* template */
var __vue_render__$1 = function() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c(
    "div",
    { class: _vm.classes },
    [
      _vm._t("default"),
      _vm._v(" "),
      _vm.withArrow
        ? _vm._t("arrow", [_c("CPopoverArrow", { class: _vm.arrowClasses })])
        : _vm._e()
    ],
    2
  )
};
var __vue_staticRenderFns__$1 = [];
__vue_render__$1._withStripped = true;

  /* style */
  const __vue_inject_styles__$1 = undefined;
  /* scoped */
  const __vue_scope_id__$1 = undefined;
  /* module identifier */
  const __vue_module_identifier__$1 = undefined;
  /* functional template */
  const __vue_is_functional_template__$1 = false;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__$1 = /*#__PURE__*/normalizeComponent(
    { render: __vue_render__$1, staticRenderFns: __vue_staticRenderFns__$1 },
    __vue_inject_styles__$1,
    __vue_script__$1,
    __vue_scope_id__$1,
    __vue_is_functional_template__$1,
    __vue_module_identifier__$1,
    false,
    undefined,
    undefined,
    undefined
  );

var script$2 = {
  name: 'TooltipContent',
  functional: true,
  render(h, context) {
    const scopedSlots = {
      default: context.scopedSlots.default,
    };

    const attrs = {
      ...context.data.attrs,
      role: 'tooltip',
    };

    const data = {
      ...context.data,
      scopedSlots,
      attrs,
    };
    return h(__vue_component__$1, data, [context.children])
  },
};

/* script */
const __vue_script__$2 = script$2;

/* template */

  /* style */
  const __vue_inject_styles__$2 = undefined;
  /* scoped */
  const __vue_scope_id__$2 = undefined;
  /* module identifier */
  const __vue_module_identifier__$2 = undefined;
  /* functional template */
  const __vue_is_functional_template__$2 = undefined;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__$2 = /*#__PURE__*/normalizeComponent(
    {},
    __vue_inject_styles__$2,
    __vue_script__$2,
    __vue_scope_id__$2,
    __vue_is_functional_template__$2,
    __vue_module_identifier__$2,
    false,
    undefined,
    undefined,
    undefined
  );

var script$3 = {
  name: 'Tooltip',
  components: { Popover },
  props: {
    noArrow: {
      type: Boolean,
      default: false,
    },
    visible: {
      type: Boolean,
      default: null,
    },
    theme: {
      type: String,
      default: null,
    },
    text: {
      type: String,
      default: null,
    },
    placement: {
      type: String,
      default: 'top',
    },
  },
  computed: {
    popoverTheme() {
      const { theme } = this;
      if (theme == null) {
        return null
      }
      return `tooltip-${this.theme}`
    },
  },
  watch: {
    visible(visible) {
      const { $_popover } = this;
      if ($_popover == null) {
        return
      }
      if (visible) {
        $_popover.show();
      } else {
        $_popover.hide();
      }
    },
    popoverTheme(popoverTheme) {
      const { $_popover } = this;
      if ($_popover == null) {
        return
      }
      $_popover.theme = popoverTheme;
    },
  },
  beforeUpdate() {
    if (this.$el == null) {
      return
    }
    this.turnListenersOff();
  },
  updated() {
    const { $el } = this;
    if ($el == null) {
      return
    }
    this.setupListeners();
    this.turnListenersOn();
  },
  destroy() {
    this.$_popover.$destory();
    this.turnListenersOff();
  },
  mounted() {
    const target = () => this.$el;
    const theme = this.popoverTheme;
    this.$_escListener = createOnEscListener(() => this.handleESC());
    const tooltipVM = this;

    this.$_popover = new Vue({
      components: { Popover },
      data() {
        return {
          theme,
          noArrow: tooltipVM.noArrow,
          visible: tooltipVM.visible,
          placement: tooltipVM.placement,
        }
      },
      methods: {
        show() {
          this.$refs.popover.show();
          this.visible = true;
        },
        hide() {
          this.$refs.popover.hide();
          this.visible = false;
        },
      },
      render(h) {
        const props = {
          target,
          withArrow: !this.noArrow,
          theme: this.theme,
          placement: this.placement,
        };
        const on = {
          'update:visible': (newVisible) => {
            if (this.visible != null) {
              return
            }
            this.visible = newVisible;
          },
        };
        const scopedSlots = {
          default(popoverProps) {
            const { text, $scopedSlots } = tooltipVM;
            if (text != null) {
              return h(__vue_component__$2, {}, [text])
            }
            return $scopedSlots.content(popoverProps)
          },
        };
        return h(Popover, { ref: 'popover', on, parent: tooltipVM, props, scopedSlots })
      },
    }).$mount();
    this.$forceUpdate();
  },
  methods: {
    setupListeners() {
      this.$_mouseEnterListener = createEventListener('mouseenter', this.$el, () => {
        this.handleMouseEnter();
      });
      this.$_mouseLeaveListener = createEventListener('mouseleave', this.$el, () => {
        this.handleMouseLeave();
      });
      this.$_blurListener = createEventListener('blur', this.$el, () => this.handleBlur());
      this.$_focusListener = createEventListener('focus', this.$el, () => this.handleFocus());
    },
    turnListenersOff() {
      this.$_mouseEnterListener != null &&
        this.$_mouseEnterListener.isOn() &&
        this.$_mouseEnterListener.off();
      this.$_mouseLeaveListener != null &&
        this.$_mouseLeaveListener.isOn() &&
        this.$_mouseLeaveListener.off();
      this.$_blurListener != null && this.$_blurListener.isOn() && this.$_blurListener.off();
      this.$_focusListener != null && this.$_focusListener.isOn() && this.$_focusListener.off();
    },
    turnListenersOn() {
      this.$_mouseEnterListener.isOn() === false && this.$_mouseEnterListener.on();
      this.$_mouseLeaveListener.isOn() === false && this.$_mouseLeaveListener.on();
      this.$_blurListener.isOn() === false && this.$_blurListener.on();
      this.$_focusListener.isOn() === false && this.$_focusListener.on();
    },
    handleMouseEnter() {
      this.$_popover.show();
      this.$_escListener.isOn() === false && this.$_escListener.on();
    },
    handleMouseLeave() {
      this.$_popover.hide();
      this.$_escListener.isOn() && this.$_escListener.off();
    },
    handleBlur() {
      this.$_popover.hide();
      this.$_escListener.isOn() && this.$_escListener.off();
    },
    handleFocus() {
      this.$_popover.show();
      this.$_escListener.isOn() === false && this.$_escListener.on();
    },
    handleESC() {
      this.$_popover.hide();
      this.$_escListener.isOn() && this.$_escListener.off();
    },
  },
  render() {
    return this.$slots.default
  },
};

/* script */
const __vue_script__$3 = script$3;

/* template */

  /* style */
  const __vue_inject_styles__$3 = undefined;
  /* scoped */
  const __vue_scope_id__$3 = undefined;
  /* module identifier */
  const __vue_module_identifier__$3 = undefined;
  /* functional template */
  const __vue_is_functional_template__$3 = undefined;
  /* style inject */
  
  /* style inject SSR */
  
  /* style inject shadow dom */
  

  
  const __vue_component__$3 = /*#__PURE__*/normalizeComponent(
    {},
    __vue_inject_styles__$3,
    __vue_script__$3,
    __vue_scope_id__$3,
    __vue_is_functional_template__$3,
    __vue_module_identifier__$3,
    false,
    undefined,
    undefined,
    undefined
  );

var index$1 = index(__vue_component__$3, __vue_component__$2);

export default index$1;
//# sourceMappingURL=index.esm.js.map
